/*
 * Copyright (c) 2016-2020 Fairphone B.V.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#define LOG_TAG "BpHiccupd"

#include "BpHiccupd.hpp"

#include <memory>

using namespace android;

namespace hiccup {
#define HICCUPD_INTERFACE_NAME "com.fairphone.IHiccupd"

IMPLEMENT_META_INTERFACE(Hiccupd, HICCUPD_INTERFACE_NAME);

sp<IHiccupd> getHiccupService() {
  sp<IServiceManager> s = defaultServiceManager();
  sp<IBinder> binder = s->getService(String16("com.fairphone.hiccupd"));
  if (binder == 0)
    return 0;
  sp<IHiccupd> hiccupd = interface_cast<IHiccupd>(binder);
  return hiccupd;
}

Vector<LogFile> *BpHiccupd::getLastKmsg() {
  Parcel data, reply;
  data.writeInterfaceToken(String16(HICCUPD_INTERFACE_NAME));
  auto v = std::make_unique<Vector<LogFile>>();
  status_t ret = remote()->transact(GET_LAST_KMESG, data, &reply);
  if (ret != OK) {
    ALOGE("Transaction failed: %i", ret);
    return v.release();
  }
  int num_logs = reply.readInt32();
  for (int i = 0; i < num_logs; i++) {
    String16 path = reply.readString16();
    int fd = dup(reply.readFileDescriptor());
    v->push_back(LogFile(path,fd));
  }
  return v.release();
}

String16 BpHiccupd::getBootReason() {
  Parcel data, reply;
  data.writeInterfaceToken(String16(HICCUPD_INTERFACE_NAME));
  status_t ret = remote()->transact(GET_BOOTREASON, data, &reply);
  if (ret != OK) {
    ALOGE("Transaction failed: %i", ret);
    return String16();
  }
  return reply.readString16();
}

String16 BpHiccupd::getPowerOnReason() {
  Parcel data, reply;
  data.writeInterfaceToken(String16(HICCUPD_INTERFACE_NAME));
  status_t ret = remote()->transact(GET_POWERONREASON, data, &reply);
  if (ret != OK) {
    ALOGE("Transaction failed: %i", ret);
    return String16();
  }
  return reply.readString16();
}

String16 BpHiccupd::getPowerOffReason() {
  Parcel data, reply;
  data.writeInterfaceToken(String16(HICCUPD_INTERFACE_NAME));
  status_t ret = remote()->transact(GET_POWEROFFREASON, data, &reply);
  if (ret != OK) {
    ALOGE("Transaction failed: %i", ret);
    return String16();
  }
  return reply.readString16();
}

int32_t BpHiccupd::getApiVersion() {
  Parcel data, reply;
  data.writeInterfaceToken(String16(HICCUPD_INTERFACE_NAME));
  status_t ret = remote()->transact(GET_API_VERSION, data, &reply);
  if (ret != OK) {
    ALOGE("Transaction failed: %i", ret);
    return -1;
  }
  return reply.readInt32();
}

} /* namespace hiccup */
