/*
 * Copyright (c) 2016-2020 Fairphone B.V.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#pragma once

/*
 * Hiccup deamon
 *
 * This deamon is the priviledged part of hiccup and exports the logcat files
 * and last_kmsg to the unpriviledged java service.
 */

#include <cstdint>

#include <utils/RefBase.h>
#include <utils/Log.h>
#include <binder/TextOutput.h>

#include <binder/IInterface.h>
#include <binder/IBinder.h>
#include <binder/Parcel.h>
#include <binder/ProcessState.h>
#include <binder/IServiceManager.h>

using namespace android;

#define HICCUP_API_VERSION 2

namespace hiccup {

class LogFile {
public:
  String16 fileName;
  int fd;
  LogFile(): fileName(""),fd(-1) {}
  LogFile(String16 path, int fd): fileName(path), fd(fd) {}
  LogFile(char *path, int fd): fileName(path), fd(fd) {}
  ~LogFile() {}
};

class IHiccupd : public IInterface {
    public:
        enum {
            GET_BOOTREASON = IBinder::FIRST_CALL_TRANSACTION,
            GET_POWERONREASON,
            GET_POWEROFFREASON,
            GET_LAST_KMESG,
            GET_API_VERSION,
        };

        virtual Vector<LogFile> *getLastKmsg() = 0;
        virtual String16 getBootReason()       = 0;
        virtual String16 getPowerOnReason()    = 0;
        virtual String16 getPowerOffReason()   = 0;
        virtual int32_t  getApiVersion()       = 0;

        // prepares the logs for the caller
        DECLARE_META_INTERFACE(Hiccupd)
};

} /* namespace hiccup */
